<?php

// Configuration
// `````````````

namespace {
	
	define('S_PAGETITLE',  'Image of the Day');	
	define('S_IMAGEPATH',  'src/');
	define('S_IMAGESTORE', dirname(__FILE__).'/htdocs/src/');
	
	define('S_PHPPATH',    '/usr/bin/php');
	
	define('DEF_REMOTE',   'ext/noremote.gif');

	define('IMGUR_API_KEY',  '');
	define('UPLOAD_TIMEOUT', 60);
	
	define('G_IMGPERPAGE', 10);
	
	define('TYPE_IMAGE',   1);
	define('TYPE_YOUTUBE', 2);
	define('TYPE_FLASH',   3);
	
	define('ALLOW_PUBLIC_UPLOAD', false);
	
	// For RSS
	define('PRIVATE_BASE_URL', 'http://10.0.0.10/');
	define('PUBLIC_BASE_URL', 'http://');
}

	
// Common functions
// ````````````````

namespace {
	
	// RENDERING
	// `````````
	
	function formatItem($item) {
		return (isLocal()) ? formatItemLocal($item) : formatItemRemote($item);
	}
	
	function formatItemLocal($item) {
		switch($item['type']) {
			case TYPE_IMAGE: {
				
				return
					'<a id="entry_'.$item['time'].'" href="'.S_IMAGEPATH.$item['data'].'">'.
						'<img class="iotd" src="'.S_IMAGEPATH.$item['data'].'" />'.
					'</a>'
				;
			} break;
			
			case TYPE_YOUTUBE: {
				return '
<div id="entry_'.$item['time'].'" class="youtube">
	<object width="425" height="349">
		<param name="movie" value="http://www.youtube-nocookie.com/v/'.$item['data'].'?version=3&amp;hl=en_GB" />
		<param name="allowFullScreen" value="true" />
		<param name="allowscriptaccess" value="always" />
		<param name="wmode" value="transparent" />
		<embed src="http://www.youtube-nocookie.com/v/'.$item['data'].'?version=3&amp;hl=en_GB" type="application/x-shockwave-flash" width="425" height="349" allowscriptaccess="always" allowfullscreen="true" wmode="transparent"></embed>
	</object>
</div>';
			} break;
			
			case TYPE_FLASH: {
				return '
					<object id="entry_'.$item['time'].'" width="600" height="400" class="iotd">
						<param name="movie" value="'.S_IMAGEPATH.$item['data'].'">
						<embed width="600" height="400" src="'.S_IMAGEPATH.$item['data'].'"></embed>
					</object>
				';
			}
			
			default: {
				return '<!-- Unknown item type -->';
			}
		}
	}
	
	function formatItemRemote($item) {
		switch($item['type']) {
			case TYPE_IMAGE: {
				if (strlen($item['remote'])) {
					return
						'<a id="entry_'.$item['time'].'" href="'.$item['remote'].'">'.
							'<img class="iotd" src="'.$item['remote'].'" />'.
						'</a>'
					;
				} else {
					return '<img id="entry_'.$item['time'].'" class="iotd" src="'.DEF_REMOTE.'" />';
				}
			}
			
			case TYPE_YOUTUBE: {
				return formatItemLocal($item);
			} break;
			
			case TYPE_FLASH: {
				return '<img id="entry_'.$item['time'].'" class="iotd" src="'.DEF_REMOTE.'"/>';
			} break;
			
			default: {
				return '';
			}
		}
	}
	
	function formatItemType($item) {
		switch( $item['type']) {
			case TYPE_IMAGE: return 'Image';
			case TYPE_YOUTUBE: return 'Youtube';
			case TYPE_FLASH: return 'Flash';
			default: return 'Unknown';
		}
	}
	
	// UPLOADING
	// `````````
	
	function manageUpload() {
		if (! isLocal()) {
			home('Uploading is only avaliable to local users.');
		}
		switch (isset($_POST['type']) ? $_POST['type'] : null) {
			case 'image': {
				manageImageUpload();
			} break;
			case 'youtube': {
				manageYoutubeUpload();
			} break;
			default: {
				home('Invalid upload request!');
			} break;
		}
	}

	// Manage image uploads
	function manageImageUpload() {
		if ( ! ($_FILES["file"]) ) home('No file selected!');
		if ($_FILES["file"]["error"] > 0) {		
			switch ($_FILES["file"]["error"]) {
				case 1:
				case 2:
					home(
						'File was too large! Max size is '.
						ini_get(upload_max_filesize).' B.'
					);
				case 4:
					home('No file selected!');
				default:
					home('Error #'.$_FILES["file"]["error"].'uploading file!');
			}
		} 
		
		if (stripos($_FILES['file']['name'], '.php') !== false) {
			home('Please remove .php from filename!');
		}
		
		// Place uploaded file in iotd-{time}.ext
		$curtime = time();
		$new_file_name = "iotd-".$curtime.".".end(explode(".",$_FILES["file"]["name"]));
		move_uploaded_file($_FILES["file"]["tmp_name"],S_IMAGESTORE.$new_file_name);

		if ($_FILES['file']['type'] == 'application/x-shockwave-flash') {
			DB\InsertItem($curtime, TYPE_FLASH, $new_file_name);
		} else {
			DB\InsertItem($curtime, TYPE_IMAGE, $new_file_name);
			
			// Spawn background imgur uploader
			callBackgroundUploader($new_file_name);
		}
		
		// Upload OK.
		home(
			"Uploaded " . $_FILES["file"]["name"] . " OK (" .
			number_format($_FILES["file"]["size"] / 1024, 2) . " Kb)"
		);
	}


	// Manage youtube uploads
	function manageYoutubeUpload() {
		if (! isset($_POST['url'])) home('Invalid Youtube URL!');
		
		$url = $_POST['url'];
		$url = str_replace(
			array(	'http://', 'https://', 'www.', 'youtube.com/', 'youtu.be/',
					'v/', 'watch?v=', '?t=', '?v='),
			array(	'', '', '', '', '', '', '', '&t=', ''),
			$url
		);
		
		DB\InsertItem(time(), TYPE_YOUTUBE, $url);

		home('Added successfully!');
	}
	
	// BACKGROUND
	// ``````````
	
	function callBackgroundUploader($iotd_file_name) {
		execInBackground(
			S_PHPPATH,
			array(dirname(__FILE__).'/bgupload.php', $iotd_file_name)
		);
	}
	
	function execInBackground($exepath, $args = array()) {
		$argsz = implode(' ', array_map('escapeshellarg', $args));
	
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
			// Windows
			
			// http://de2.php.net/manual/en/function.exec.php#35731
			if (! file_exists($exepath)) return;
			pclose(popen(
				'start "'.S_PAGETITLE.'" "'.$exepath.'" '.$argsz,
				"r"
			));
			
		} else {
			// Linux
			exec($exepath.' '.$argsz.' > /dev/null 2>&1 &'); 
			
		}
		
	}
	
	// HELPERS
	// ````````
	
	function isLocal() {
		return (strpos($_SERVER['REMOTE_ADDR'], '10.0.0.') === 0);
	}
	
	function hesc($sz) {
		return htmlentities($sz, ENT_QUOTES, 'UTF-8');
	}
	
	function redirect($url) {
		header('Location: '.$url);
		die();
	}
	
	function fancyDie($sz) {
		header('Content-Type: text/html; charset=UTF-8');
		die('<!DOCTYPE html><div style="font-size:16px;margin:16px auto;padding:4px 32px;width:300px;text-align:center;background:#EEE;border:1px solid #444;border-top:1px solid #DDD;border-left:1px solid #DDD;">'.$sz.'</div>');
	}
		
	function home($message='') {
		fancyDie($message.'<br><a href="javascript:;" onclick="history.back();">&laquo; Return</a>');
	}
}

// Database functions
// ``````````````````

namespace DB {

	$db = new \PDO('sqlite:'.dirname(__FILE__).'/motd.db');
	$db->query('
		CREATE TABLE IF NOT EXISTS items (
			id     INTEGER PRIMARY KEY,
			time   INTEGER,
			type   INTEGER,
			data   TEXT,
			remote TEXT
		);
	');

	function InsertItem($time, $type, $data) {
		global $db;
		$stmt = $db->prepare(
			'INSERT INTO items (time, type, data, remote) VALUES (?, ?, ?, ?)'
		);
		$stmt->execute(array($time, $type, $data, ''));
	}

	function setRemoteData($filename, $remote) {
		global $db;
		$stmt = $db->prepare('UPDATE items SET remote=? WHERE data=?');
		$stmt->execute(array($remote, $filename));
	}
	
	function getLatest() {
		global $db;
		$stmt = $db->query('SELECT * FROM items ORDER BY time DESC LIMIT 1');
		$result = $stmt->fetchAll();
		if (! count($result)) return array('type'=>0);
		return $result[0];
	}
	
	function getRange($start, $count) {
		global $db;
		$stmt = $db->query(
			'SELECT * FROM items ORDER BY time DESC LIMIT '.
			intval($start).', '.intval($count)
		);
		return $stmt->fetchAll();
	}
	
	function getCount() {
		global $db;
		$stmt = $db->query('SELECT COUNT("id") c FROM items LIMIT 1');
		$results = $stmt->fetchAll();
		return $results[0]['c'];
	}

}